@@ -168,11 +168,15 @@ GGDC10S %>% fsubset(Variable == "VA", Country, Year, AGR, SUM) %>%
              head
 
 ## -------------------------------------------------------------------------------------------------
-# This replaces variables mpg, carb and wt by their log
-mtcars %>% ftransform(fselect(., mpg, carb, wt) %>% lapply(log)) %>% head
+# This replaces variables mpg, carb and wt by their log (.c turns expressions into character vectors)
+mtcars %>% ftransformv(.c(mpg, carb, wt), log) %>% head
 
-# This adds the log of mpg, carb and wt
-mtcars %>% ftransform(fselect(., mpg, carb, wt) %>% lapply(log) %>% add_stub("log.")) %>% head
+# Logging numeric variables
+iris %>% ftransformv(is.numeric, log) %>% head
+
+## -------------------------------------------------------------------------------------------------
+# Logging values and replacing generated Inf values
+mtcars %>% ftransform(fselect(., mpg, cyl, vs:gear) %>% lapply(log) %>% replace_Inf) %>% head
 
 ## -------------------------------------------------------------------------------------------------
 GGDC10S %>% fsubset(Variable == "VA", Country, Year, AGR, SUM) %>%
@@ -205,6 +209,10 @@ GGDC10S %>%
 settransform(GGDC10S, AGR_gmed = AGR > fmedian(AGR, list(Variable, Country), TRA = "replace"))
 tail(GGDC10S, 3)
 
+# Dividing (scaling) the sectoral data (columns 6 through 16) by their grouped standard deviation
+settransformv(GGDC10S, 6:16, fsd, list(Variable, Country), TRA = "/", apply = FALSE)
+tail(GGDC10S, 3)
+
 
 ## -------------------------------------------------------------------------------------------------
 # This subtracts weighted group means from the data, using SUM column as weights.. 
